*------------------------------------------------------------*
*Program: TNDS Workshop: SAS Program
*Programmer: Emily Sanchez emily.sanchez@tufts.edu
*Date: 19 January 2021
*Purpose: TNDS Workshop: Importing & Saving Data
*Last edited: 23 February 2021
*-------------------------------------------------------------*;

*******************************************************
*******************************************************
*****CREATE LIBRARIES*****;
LIBNAME IN 'C:\Users\esanch09\Desktop\data for workshop'; 
*OPTIONS FMTSEARCH=(IN); *<-- **IF YOU'RE IMPORTING FILES WITH FORMATS, YOU NEED TO USE 'OPTIONS FMTSEARCH=(LIBRARY REFERENCE/DEFAULT IS WORK)' TO TELL SAS WHERE THE FORMAT FILE IS LOCATED;
*REVIEW LOG & EXPLORER/IN LIBRARY*

*******************************************************
*******************************************************
******READING IN XLSX DATA******;
*USE PROC STATEMENT;
PROC IMPORT OUT= WORKSHOP1
            DATAFILE= 'C:\Users\esanch09\Desktop\data for workshop\workshop_data_numeric.xls' 
            DBMS=XLS REPLACE;  *<- TELLS SAS WHAT TYPE OF FILE 
									TO READ (COULD ALSO BE CSV, TXT FILE FOR EXAMPLE);
     		GETNAMES=YES; *<- TELLS SAS WHETHER TO USE THE FIRST LINE 
								OF THE INPUT FILE FOR VARIABLE NAMES;
     		DATAROW=2; *<- WHERE SAS SHOULD START READING DATA;
RUN;
*REVIEW LOG & EXPLORERE/WORK LIBRARY;

/* IF IMPORTING A SAS FILE (SAS7BDAT), CAN DO SO:
DATA WORKSHOP1;
SET IN.WORKSHOP_DATA; *IN <-- IS THE LIBRARY NAME// WORKSHOP_DATA IS THE SAS FILE WE WISH TO IMPORT;
RUN;
*REVIEW LOG AND CONFIRM SUCCESSFUL IMPORT BY EXPLORING LIBARIES*/

*******************************************************
*******************************************************
******DESCRIBE, VIEW AND EDIT DATA ONCE IMPORTED*****;

PROC CONTENTS DATA=WORKSHOP1; RUN;
*REVIEW LOG & RESULTS.  CAN SEE THE NUMBER OF OBSERVATIONS (N=2653), NUMBER OF VARIABLES (14), LIST OF VARIABLES AND ATTRIBUTES (VARIABLE NAME, TYPE, LENGTH, FORMAT AND LABEL);

*****FIND ALL UNIQUE VALUES OF GENDER******;
PROC FREQ DATA=WORKSHOPS;
TABLES GENDER/MISSING;
TITLE 'GENDER FREQUENCY TABLE WITH MISSING VALUES';
RUN;
*REVIEW LOG & RESULTS.  CAN SEE UNIQUE VALUES INCLUDE -99, 0 AND 1.

*****FIND THE NUMBER OF UNIQUE VALUES OF AGE IN DAYS******;
PROC FREQ DATA=WORKSHOP1 NLEVELS;
TABLES AGEDAYS/MISSING;
TITLE 'EXPLORING UNIQUE VALUES OF AGEDAYS';
RUN;
*REVIEW LOG & RESULTS.  CAN SEE THERE ARE 690 LEVELS OR UNIQUE VAULES FOR AGE IN DAYS AND 1 MISSING LEVEL.

*****FIND THE NUMBER OF ROWS WITH COMPLETE OBSERVATIONS******;
DATA WORKSHOP2;
SET WORKSHOP1;
MISS_N = CMISS(OF STUDY_ID -- HFIAS_CAT);
*CMISS COUNTS THE NUMBER OF ARGUMENTS MISSING FOR BOTH CHARACTER AND NUMERIC VARIABLES;
RUN;
*REVIEW LOG;
PROC PRINT DATA=WORKSHOP2 (OBS=20);
RUN;
*REVIEW LOG & RESULTS.  CAN SEE MISS_N IS A NEW COLUMN/VARIABLE IN WORKSHOP2.;

*USE PROC FREQ TO SEE N_MISS VALUES AND FREQUENCIES.;
PROC FREQ DATA=WORKSHOP2;
TABLES MISS_N/MISSING;
TITLE 'USING N_MISS TO COUNT OBSERVATIONS MISSING VALUES';
RUN;
*REVIEW LOG & RESULTS.  A TOTAL OF 2623 OBSERVATIONS ARE COMPLETE/FREE OF MISSING DATA.;

*****THE NUMBER OF MISSING OBSERVATIONS WITH EDUCATION*****;
*CAN THEN USE PROC FREQ TO SEE NUMBER OF OBSERVATIONS MISSING EDUCATION.;
PROC FREQ DATA=WORKSHOP2;
TABLES EDU*MISS_N;
TITLE 'NUMBER OF OBSERVATIONS MISSING EDUCATION VALUE';
RUN;
*REVIEW LOG & RESULTS.

IDENTIFYING THE NUMBER OF MISSING OBSERVATIONS FOR CONTINUOUS DATA WILL BE DEMONSTRATED LATER;

*******************************************************
*******************************************************
*****GENERATE AND LABEL VARIABLES AND OBSERVATIONS*****;

*****CREATING/DEFINING NEW VARIABLES*****
NEW_VARIABLE = 10/ 'TEN'/ OLD_VARIABLE

IF DATA IS MISSING ON "EXPRESSION" SIDE, THE NEW VARIABLE WILL ALSO BE MISSING:
IF OLD_VARIABLE =. THEN NEW_VARIABLE WILL ALSO BE MISSING;

DATA WORKSHOP3;
SET WORKSHOP2;
AGEYEARS = AGEDAYS/365.25;
RUN;
*REVIEW LOG;
PROC PRINT DATA=WORKSHOP3 (OBS=20); 
VAR AGEDAYS AGEYEARS; 
RUN;
*REVIEW LOG & RESULTS TO SEE NEWLY CREATED VARIABLE AGEYEARS.

*ANOTHER WAY TO CREATE NEW VARIABLES IS TO USE IF/THEN STATEMENTS*;
DATA WORKSHOP3;
SET WORKSHOP3;
IF HFIAS_SCORE <1 THEN HFIAS_CAT_NEW=4;
ELSE IF 1=<HFIAS_SCORE<10 THEN HFIAS_CAT_NEW=3;
ELSE IF 10=<HFIAS_SCORE<14 THEN HFIAS_CAT_NEW=2;
ELSE IF 14=<HFIAS_SCORE THEN HFIAS_CAT_NEW=1;
ELSE IF HFIAS_CAT_NEW=.;
RUN;
*REVIEW LOG;
PROC PRINT DATA=WORKSHOP3 (OBS=20); RUN;
*REVIEW LOG & RESULTS;

DATA WORKSHOP3;
SET WORKSHOP3 (DROP =HFIAS_CAT);
RUN;
*REVIEW LOG;
PROC PRINT DATA=WORKSHOP3 (OBS=20); RUN;
*REVIEW LOG & RESULTS;

*****LABELING*****;

DATA WORKSHOP3;
SET WORKSHOP3;
LABEL AGEYEARS = 'Age in Years'
	  AGEMONTHS = 'Age in Months'
	  AGEDAYS = 'Age in Days'
	  EDU = 'Education';
RUN;
*REVIEW LOG;
PROC PRINT DATA=WORKSHOP3 (OBS=20) LABEL;
RUN;
*REVIEW LOG & RESULTS;

PROC FREQ DATA=WORKSHOP3;
TABLES EDU;
RUN;
*REVIEW LOG & RESULTS. LABEL NAME APPLIED TO EDU, NO NEED FOR LABEL STATEMENT HERE.

*COMPARISON BEFORE/AFTER;
PROC CONTENTS DATA=WORKSHOP1; RUN; *REVIEW LOG & RESULTS;
PROC CONTENTS DATA=WORKSHOP3; RUN; *REVIEW LOG & RESULTS;
*REVIEW LIST OF VARIABLES AND ATTRIBUTES TABLES FROM EACH DATAFILE, SEE CHANGES UNDER 'LABEL';

*****FORMATTING*****

FORMAT statement - HOW SAS VARIABLES ARE DISPLAYED AND TO SPECIFY LEVELS OF CATEGORICAL VARIABLES:

FORMAT TYPE:	FORMAT:		ORIGINAL:		RESULT:
NUMERIC			5.1			125.36			125.3   <--- DOES NOT ROUND BUT TRUNCATES DATA
CHARACTER		$6.			CHICKEN			CHICKE
DATE			DATE9.		JUNE 1, 1972	01JUNE1972  <--- SAS STORES DATES, TIMES AND DATELINES AS NUMBERS,
															DATES ARE COUNTED IN DAYS FROM A ZERO POINT OF JANUARY 1 1960;

PROC PRINT DATA=WORKSHOP3;
VAR AGEMONTHS;
FORMAT AGEMONTHS 5.1;
RUN;
*REVIEW LOG & RESULTS.  AGEMONTHS ONLY FORMATTED IN PROC STEP;

PROC PRINT DATA=WORKSHOP3;
VAR AGEMONTHS;
*FORMAT AGEMONTHS 5.1;
RUN;
*REVIEW LOG & RESULTS.  YOU CAN ALSO FORMAT IN YOUR DATASET.;

DATA WORKSHOP3;
SET WORKSHOP3;
FORMAT AGEMONTHS 5.1;
RUN;
*REVIEW LOG;

PROC CONTENTS DATA=WORKSHOP3; RUN;
*REVIEW LOG AND RESULTS;
PROC PRINT DATA=WORKSHOP3 (OBS=20); RUN;
*REVIEW LOG AND RESULTS;

*USING PROC FORMAT TO DEFINE YOUR FORMAT*
THIS IS INDEPENDENT OF YOUR DATASET AND MUST BE ASSIGNED/ASSOCIATED WITH A SUBSEQUENT DATASTEP AND/OR PROC.
PROC FORMAT CREATES A MAP OF DATA VALUES INTO DATA LABELS.*;
PROC FORMAT;
VALUE GENDERF 0='Female'  
			  1='Male'
			  -99 = 'Not Documented'; *VARIABLEF IS A GREAT WAY TO LABEL YOUR FORMATS.  ADD IN SPACES FOR MALE SO THAT THE TWO STRINGS ARE OF THE SAME LENGTH;
VALUE EDUF 	  0='None'  
			  1='Some Primary'
			  2='Completed Primary'
			  3='Some Secondary'
			  4='Completed Secondary'
			  5='More than secondary'
			  -99 = 'Not Documented';
RUN;
*REVIEW LOG.;

/*ABOVE WAS AN EXAMPLE USING A NUMERIC VARIABLE.  CHARACTER VARIABLES CAN ALSO BE FORMATTED:
PROC FORMAT;
VALUE $ GENDER_CHAR 'F'='Female'  
			    'M'='Male'
			    'N/A' = 'Not Documented';
RUN;

IF YOU ARE ANALYZING DATA THAT HAS ALREADY BEEN FORMATTED, YOU NEED TO HAVE THE FORMAT CATAOLG/LIBRARY*/

***ASSOCIATE THE FORMAT WITH WITH DATA SET/DATA STEP***;
DATA WORKSHOP3;
SET WORKSHOP3;
FORMAT GENDER GENDERF.;
RUN;
*REVIEW LOG;

PROC PRINT DATA=WORKSHOP3 (OBS=20); RUN; *REVIEW LOG & RESULTS;
PROC CONTENTS DATA=WORKSHOP3; RUN; *REVIEW LOG & RESULTS.  CAN SEE TYPE OF VARIABLE HAS CHANGED TO CHARACTER.;


*******************************************************
*******************************************************
*****SUMMARY STATISTICS*****;

*CATEGORICAL VARIABLES*; 

PROC FREQ DATA=WORKSHOP3;
TABLES GENDER EDU/MISSING;
FORMAT EDU EDUF.;
RUN;
*REVIEW LOG & RESULTS;

*CONTINUOUS VARIABLES*;
PROC UNIVARIATE DATA=WORKSHOP3;
VAR AGEDAYS;
RUN;
*REVIEW LOG & RESULTS*;

/* ALTERNATIVE, USE PROC MEANS;
PROC MEANS DATA=WORKSHOP3 N NMISS MEAN STD MEDIAN MODE MIN MAX RANGE P10 P25 P50 P75 P90 SKEWNESS KURTOSIS MAXDEC=2; *N MEAN STD MIN MAX ARE DEFAULT;
VAR AGEDAYS AGEMONTHS AV_WEIGHT AV_LEN AV_MUAC _ZWEI _ZWFL HFIAS_SCORE;
RUN; */

*SAVE YOU OUTPUT AS A PERMANENT SAS FILE;
DATA IN.PERM_FILE;
SET WORKSHOP3;
RUN;

**DON'T FORGET TO PERMANENTLY SAVE YOUR FORMATS*;
PROC CATALOG CAT=WORK.FORMATS;
COPY OUT=IN.FORMATS;
RUN; QUIT; *<-- USING QUIT.

*REVIEW LOG & OUT LIBARAY/FOLDER.
SAVING YOUR FORMATS WITH YOUR PERMANENT SAS FILE IS CRUCIAL!  OTHERWISE WILL NOT BE ABLE
TO OPEN DATASET IN NEW SESSION WITH FORMATS!

*SAVING AS AS XLS*;
PROC EXPORT DATA=WORKSHOP3 DBMS=XLS
OUTFILE='C:\Users\esanch09\Desktop\data for workshop\saved_sas_xls' 
REPLACE;
RUN;
*REVIEW LOG;
****HOW DO WE PRESRVE FORMATS WHEN EXPORTING TO XLS?***
 THERE IS NO EASY WAY TO DO THIS.  YOU WOULD HAVE TO USE A MACRO
OR WRITE A SAS FORMAT AGAIN WITH CODE --
THIS IS WHY IT IS SO IMPORTANT TO KEEP PERMANENT SAS DATA SETS!
SO YOU DON'T HAVE TO RUN THROUGH ALL THE CODE AGAIN.

*******************************************************
*******************************************************
*****PRODUCING HISTOGRAMS TO INSPECT VARIABLESS*****;

PROC SGPLOT DATA=WORKSHOP3;
HISTOGRAM HFIAS_SCORE;
TITLE 'HISTOGRAM OF HFIAS SCORE';
RUN;
*REVIEW LOG & RESULTS;

PROC UNIVARIATE DATA=WORKSHOP3;
HISTOGRAM HFIAS_SCORE;
RUN;
*REVIEW LOG & RESULTS;

*SIMPLE UNPAIRED T-TEST*;

PROC FREQ DATA=WORKSHOP3; 
TABLES GENDER;
TITLE 'GENDER FREQUENCY TABLE';
RUN;
*REVIEW LOG & RESULTS;

DATA WORKSHOP4;
SET WORKSHOP3;
IF GENDER=-99 THEN GENDER=.;
RUN;
*REVIEW LOG;

PROC TTEST DATA=WORKSHOP4 ALPHA=0.5;
VAR AV_LEN; *DEPENDENT VARIABLE;
CLASS GENDER;
RUN;
*REVIEW LOG & RESULTS;

*SIMPLE WILCOXON RANK SUM*;
PROC NPAR1WAY DATA=WORKSHOP4;
VAR AV_LEN;
CLASS GENDER;
RUN;
*REVIEW LOG & RESULTS;

*******************************************************
*******************************************************
*****MEASURES OF ASSOCIATION (PEARSON VS. SPEARMAN)*****;
PROC CORR DATA=WORKSHOP4 PEARSON;
VAR AV_WEIGHT AV_LEN;
RUN;
*REVIEW LOG & RESULTS;

PROC CORR DATA=WORKSHOP4 SPEARMAN;
VAR AV_WEIGHT AV_LEN;
RUN;
*REVIEW LOG & RESULTS;

*******************************************************
*******************************************************
*****SIMPLE LINEAR REGRESSION*****;
PROC REG DATA=WORKSHOP4;
MODEL AV_LEN = AV_WEIGHT/CLI;
RUN;
*REVIEW LOG & RESULTS;

*******************************************************
*******************************************************
*****MULTIVARIABLE LINEAR REGRESSION*****;
PROC CORR DATA=WORKSHOP4 PEARSON;
VAR AV_LEN AV_WEIGHT HFIAS_SCORE AV_MUAC;
RUN;
*REVIEW LOG & RESULTS;

PROC REG DATA=WORKSHOP4;
MODEL AV_LEN = AV_WEIGHT GENDER EDU HFIAS_SCORE AV_MUAC/VIF;
RUN; QUIT;
*REVIEW LOG & RESULTS;

